#include <signal.h>
#include <stdio.h>
#include <unistd.h>

#include "./alsa_midi.h"


static
int jack_process(jack_nframes_t nframes, void *arg)
{
	alsa_midi_t *midi = arg;
	(midi->read)(midi, nframes);
	(midi->write)(midi, nframes);
	return 0;
}

static
void jack_freewheel(int starting, void *arg)
{
	alsa_midi_t *midi = arg;
	if (starting)
		(midi->stop)(midi);
	else
		(midi->start)(midi);
}

static
int keep_walking = 1;

static
void sigint_handler(int i)
{
	keep_walking = 0;
}

static
void jack_shutdown(void *arg)
{
	keep_walking = 0;
}

int main(int argc, char *argv[])
{
	alsa_midi_t *midi;
	//jack_status_t status;
	//jack_client_t *jack = jack_client_open("alsa_midi", JackServerName|JackNoStartServer, &status, "dummy");
	jack_client_t *jack = jack_client_new("alsa_midi");

	if (!jack)
		goto fail0;

	//midi = alsa_rawmidi_new(jack);
	midi = alsa_seqmidi_new(jack);
	if (!midi)
		goto fail1;

	jack_set_process_callback(jack, jack_process, midi);
	jack_set_freewheel_callback(jack, jack_freewheel, midi);
	jack_on_shutdown(jack, jack_shutdown, NULL);

	signal(SIGINT, &sigint_handler);
	signal(SIGTERM, &sigint_handler);

	(midi->attach)(midi);
	(midi->start)(midi);

	if (jack_activate(jack)) {
		printf("can't activate jack client\n");
		goto fail2;
	}

	while (keep_walking)
		sleep(1);

	jack_deactivate(jack);
fail2:
	(midi->stop)(midi);
	(midi->detach)(midi);
	(midi->destroy)(midi);
fail1:
	jack_client_close(jack);
fail0:
	return 0;
}
